/*
Copyright 2010 Zephyr Renner.

This file is part of PanoSalado.

PanoSalado is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PanoSalado is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PanoSalado. If not, see <http://www.gnu.org/licenses/>.
*/
package com.panosalado.model{
	
	import com.panosalado.events.ReadyEvent;
	import com.panosalado.loading.IOErrorEventHandler;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.Dictionary;
	
	public class DeepZoomTilePyramid extends TilePyramid {
		
		public static var urlPattern:RegExp = /(.*)(_f|front)\.(xml|XML)$/;
		
		public var tierOffset:int;
		
		public function DeepZoomTilePyramid():void {
			super();
		}
		
		override public function loadDescriptor(params:Params, paths : Dictionary):void
		{ 
			/*
				<?xml version="1.0" encoding="utf-8"?>
				<Image TileSize="256" Overlap="1" Format="jpg" ServerFormat="Default" xmnls="http://schemas.microsoft.com/deepzoom/2009">
				<Size Width="5885" Height="5885" />
				</Image>
			*/
			width = int(params.width); 
			height = int(params.height);
			overlap = int(params.overlap);
			format = 'jpg';
			tileSize = int(params.tileSize);
			this.paths = paths;
			
			descriptor = params;
				
			deriveProperties(width, height, tileSize);
				
			dispatchEvent(new ReadyEvent(ReadyEvent.READY, this));
		}
			
		override public function clone(into:TilePyramid = null):TilePyramid {
			var clone:DeepZoomTilePyramid = 
				(into != null && into is DeepZoomTilePyramid) ? into as DeepZoomTilePyramid : new DeepZoomTilePyramid();
			clone.width = width;
			clone.height = height;
			clone.numTiles = numTiles;
			clone.tileSize = tileSize;
			clone.overlap = overlap;
			clone.numTiers = numTiers;
			clone.widths = widths;
			clone.heights = heights;
			clone.accurateWidths = accurateWidths;
			clone.accurateHeights = accurateHeights;
			clone.columns = columns;
			clone.rows = rows;
			clone.paths = paths;
			clone.format = format;
			
			clone.tierOffset = tierOffset;
			
			return clone;
		}
		
		override public function disassembleDescriptorURL(path:String):DisassembledDescriptorURL {
			//e.g: base/path/name_f.xml
			var o:Object = urlPattern.exec(path);
			var base:String = o[1];
			var id:String = o[2];
			var extension:String = o[3];
			
			return new DisassembledDescriptorURL(base, id, extension);
		}
		
		override public function getTileURL(t:uint, c:uint, r:uint):String {
			t += tierOffset;
	  
			var path:String = paths[String( t ) + "_" + String( c ) + "_" + String( r ) + "." + format];
			  
			if (!path)
				return "";
				
			return path;
		}
		
		protected function getMaximumLevel(width:uint, height:uint):uint {
			return Math.ceil(Math.log(Math.max(width, height)) / Math.LN2)
		}
		
		protected function deriveProperties(width:uint, height:uint, tileSize:uint):void {
			numTiers = int(getMaximumLevel(width, height));
			
			var columns:uint;
			var rows:uint;
			var widthsArray:Array = new Array();
			var heightsArray:Array = new Array();
			var columnsArray:Array = new Array();
			var rowsArray:Array = new Array();
			var accurateWidthsArray:Array = new Array();
			var accurateHeightsArray:Array = new Array();
			var accurateWidth:Number = width;
			var accurateHeight:Number = height;
			
			numTiles = 0;
			
			for(var t:int = numTiers; t >= 0; t--){
				// compute number of rows & columns
				columns = Math.ceil(width / tileSize);
				rows = Math.ceil(height / tileSize);
				
				widthsArray[t] = width;
				heightsArray[t] = height;
				columnsArray[t] = columns;
				rowsArray[t] = rows;
				accurateWidthsArray[t] = accurateWidth;
				accurateHeightsArray[t] = accurateHeight;
				
				numTiles += columns * rows;
				
				if (width <= tileSize && height <= tileSize) break;
				
				// compute dimensions of next level
				width = (width + 1) >> 1;
				height = (height + 1) >> 1;
				accurateWidth *= 0.5;
				accurateHeight *= 0.5;
			}
			tierOffset = t;
			widthsArray.splice(0, tierOffset);
			heightsArray.splice(0, tierOffset);
			accurateWidthsArray.splice(0, tierOffset);
			accurateHeightsArray.splice(0, tierOffset);
			columnsArray.splice(0, tierOffset);
			rowsArray.splice(0, tierOffset);
			
			numTiers -= tierOffset;
			
			widths = Vector.<int>(widthsArray);
			heights = Vector.<int>(heightsArray);
			accurateWidths = Vector.<Number>(accurateWidthsArray);
			accurateHeights = Vector.<Number>(accurateHeightsArray);
			columns = Vector.<int>(columnsArray);
			rows = Vector.<int>(rowsArray);
		}
	}
}